Attribute VB_Name = "FillHelsinkiOccurrence"
Option Compare Database
Option Explicit

' Procedure for filling the HelsinkiOccurrence table
Sub Main()
    Dim con As New ADODB.Connection
    Dim rAccepted, rAreas, rOccurr As ADODB.Recordset
    Dim rSuperAreas, rSubAreas As ADODB.Recordset
    Dim sql As String
    Dim sqlSnippet As String
    
    Set con = CurrentProject.Connection
    
    ' first delete everything from the helsinki occurrence table
    DoCmd.RunSQL ("DELETE HelsinkiOccurrence FROM HelsinkiOccurrence")
    
    Set rAccepted = New ADODB.Recordset
    sql = "SELECT RIdentifier, PTNameFk, PTRefFk FROM v_em_helsinkiaccepted"
    rAccepted.Open sql, con, adOpenStatic, adLockReadOnly
    ' step through all accepted taxa to be exported to Helsinki
    Do Until rAccepted.EOF
        ' for each accepted taxon step through list of distinct areas
        Set rAreas = New ADODB.Recordset
        
        sql = "SELECT DISTINCT AreaId, EmCode FROM emOccurrence INNER JOIN emArea ON emOccurrence.AreaFk = emArea.AreaId "
        sql = sql & "WHERE emOccurrence.PTNamefk = " & rAccepted("PTNameFk") & " AND emOccurrence.PTRefFk = " & rAccepted("PTRefFk")
      
        rAreas.Open sql, con, adOpenStatic, adLockReadOnly
        Do Until rAreas.EOF
            ' Now get all records for the given area and insert the first one into table HelsinkiAccepted
            ' The records are sorted by CalculatedFlag so that calculated records are inserted with a priority
            ' Added 17.10.2007: if we have more than 1 calculated records, then take the records with the highest status value (simply done by adding the sort criterion)
            Set rOccurr = New ADODB.Recordset
            
            sql = "SELECT * FROM emOccurrence WHERE (emOccurrence.PTNamefk = " & rAccepted("PTNameFk") & ") AND (emOccurrence.PTRefFk = " & rAccepted("PTRefFk") & ") AND "
            sql = sql & "(SummaryStatus IS NOT NULL) AND (AreaFk = " & rAreas("AreaId") & ") ORDER BY CalculatedFlag DESC, SummaryStatus DESC"
            
            rOccurr.Open sql, con, adOpenStatic, adLockReadOnly
            ' now insert the first record
            If (rOccurr.RecordCount > 0) Then
                sql = "INSERT HelsinkiOccurrence (OccurrenceId, AcceptedTaxonFk, AreaCode, SummaryStatus, Native, Introduced, Cultivated, CalculatedFlag) "
                sql = sql & "VALUES (" & rOccurr("OccurrenceId") & ", " & rAccepted("RIdentifier") & ", '" & Trim(rAreas("emCode")) & "', " & rOccurr("SummaryStatus")
                sql = sql & ", '" & rOccurr("Native") & "', '" & rOccurr("Introduced") & "', '" & rOccurr("Cultivated") & "', "
                If (rOccurr("CalculatedFlag") = "falsch") Then
                    sql = sql & "0)"
                Else
                    sql = sql & "1)"
                End If
                DoCmd.RunSQL (sql)
            End If
            rOccurr.Close
            rAreas.MoveNext
        Loop
        rAreas.Close
        Debug.Print (rAccepted("RIdentifier"))
        rAccepted.MoveNext
    Loop
    rAccepted.Close
    Debug.Print ("")
    Debug.Print ("Now: Clear Superareas ...")
    ' Clean HelsinkiOccurrence table so that superareas are removed if subareas with identical status value exist.
    ' The definition of super- and subareas is:
    ' HS <-- HS(A), HS(G), HS(S)
    ' GA <-- GA(F), GA(C)
    ' JU <-- BH, CG, CT, MK, SL, SR
    ' AE <-- AE(G), AE(T)
    ' LS <-- LE, SY
    ' IJ <-- IR, JO
    ' AB <-- AB(A), AB(N)
    ' TCS <-- AB, AR, GG
    ' Cz <-- Cs, Sk
    ' Md <-- Md(D), Md(M), Md(P)
    ' Bl <-- Bl(I), Bl(M), Bl(N)
    ' Ca <-- Ca(C), Ca(F), Ca(G), Ca(H), Ca(L), Ca(P), Ca(T)
    ' Az <-- Az(C), Az(F), Az(G), Az(J), Az(L), Az(M), Az(P), Az(S), Az(T)
    ' Hb <-- Hb(E), Hb(N)
    ' Rf <-- Rf(A), Rf(C), Rf(CS), Rf(E), Rf(K), Rf(N), Rf(NW), Rf(S)
    ' Tu <-- Tu(A), Tu(E)
    ' Uk <-- Uk(K), Uk(U)
    ' Si <-- Si(S), Si(M) Hinzugefgt von Dominik 22_11_2013
    ' Be <-- Be(B), Be(L) Hinzugefgt von Dominik 22_11_2013

    
    Set rSuperAreas = New ADODB.Recordset
    sql = "SELECT * FROM HelsinkiOccurrence WHERE AreaCode IN ('AB', 'TCS', 'HS', 'GA', 'JU', 'AE', 'LS', 'IJ', 'CZ', 'MD', 'BL', 'CA', 'AZ', 'HB', 'RF', 'TU', 'UK')"
    rSuperAreas.Open sql, con, adOpenStatic, adLockReadOnly
    Do Until rSuperAreas.EOF
    
        Select Case rSuperAreas!AreaCode
            Case "HS"
                sqlSnippet = " AND AreaCode IN ('HS(A)', 'HS(G)', 'HS(S)')"
            Case "GA"
                sqlSnippet = " AND AreaCode IN ('GA(F)', 'GA(C)')"
            Case "JU"
                sqlSnippet = " AND AreaCode IN ('BH', 'CG', 'CT', 'MK', 'SL', 'SR')"
            Case "AE"
                sqlSnippet = " AND AreaCode IN ('AE(G)', 'AE(T)')"
            Case "LS"
                sqlSnippet = " AND AreaCode IN ('LE', 'SY')"
            Case "IJ"
                sqlSnippet = " AND AreaCode IN ('IR', 'JO')"
            Case "AB"
                sqlSnippet = " AND AreaCode IN ('AB(A)', 'AB(N)')"
            Case "TCS"
                sqlSnippet = " AND AreaCode IN ('AB', 'AR', 'GG')"
            Case "CZ"
                sqlSnippet = " AND AreaCode IN ('CS', 'SK')"
            Case "MD"
                sqlSnippet = " AND AreaCode IN ('Md(D)', 'MD(M)', 'Md(P)')"
            Case "BL"
                sqlSnippet = " AND AreaCode IN ('BL(I)', 'BL(M)', 'BL(N)')"
            Case "CA"
                sqlSnippet = " AND AreaCode IN ('CA(C)', 'CA(F)', 'CA(G)', 'CA(H)', 'CA(L)', 'CA(P)', 'CA(T)')"
            Case "AZ"
                sqlSnippet = " AND AreaCode IN ('AZ(C)', 'AZ(F)', 'AZ(G)', 'AZ(J)', 'AZ(L)', 'AZ(M)', 'AZ(P)', 'AZ(S)', 'AZ(T)')"
            Case "HB"
                sqlSnippet = " AND AreaCode IN ('HB(E)', 'HB(N)')"
            Case "RF"
                sqlSnippet = " AND AreaCode IN ('RF(A)', 'RF(C)', 'RF(CS)', 'RF(E)', 'RF(K)', 'RF(N)', 'RF(NW)', 'RF(S)')"
            Case "TU"
                sqlSnippet = " AND AreaCode IN ('TU(A)', 'TU(E)')"
            Case "UK"
                sqlSnippet = " AND AreaCode IN ('UK(K)', 'UK(U)')"
            Case "SI"
                sqlSnippet = " AND AreaCode IN ('SI(S)', 'SI(M)')"
            Case "BE"
                sqlSnippet = " AND AreaCode IN ('BE(B)', 'BE(L)')"
        
        End Select
                
        Set rSubAreas = New ADODB.Recordset
        sql = "SELECT COUNT(*) AS Count FROM HelsinkiOccurrence WHERE (AcceptedTaxonFk = " & rSuperAreas!AcceptedTaxonFk & ") AND (SummaryStatus = " & rSuperAreas!SummaryStatus & ")"
        sql = sql & sqlSnippet
        rSubAreas.Open sql, con, adOpenStatic, adLockReadOnly
        If rSubAreas!Count > 0 Then
            DoCmd.RunSQL ("DELETE HelsinkiOccurrence FROM HelsinkiOccurrence WHERE OccurrenceId = " & rSuperAreas!OccurrenceId)
        End If
        rSubAreas.Close
        rSuperAreas.MoveNext
    Loop
    rSuperAreas.Close
    con.Close
    Debug.Print ("Done. Have a nice day!")
End Sub
